program Galilee

  use sep
  use lint2
  use leakint
  use lapfac
  use igrad1
  use polydiv
  use irls_priv
  use cgstep_mod
  use solver_irls
  use track_modeling_mod
  use lint_helderiv_mod

  implicit none

  real, dimension(:)   , pointer                :: x1,x2
  integer			                :: n1,n2,nd,m12
  integer				        :: mn1,mn2
  real					        :: mo1,mo2,md1,md2,l1cst
  integer                                       :: niter,iter,stat,nfreq,nmem
  real, dimension(:)   , allocatable,target     :: x
  real, dimension(:,:) , pointer	        :: input,xy
  real, dimension(:)   , pointer	        :: z, rd, rm 
  logical, parameter			        :: T=.true.,F=.false.

  integer                                       :: ndim, nreg, inv
  integer, dimension(:), pointer                :: n, m, mn

  real                                          :: rho, tau
  type(filter)                                  :: aa
  real                                          :: lapeps
  integer                                       :: ncoef
  real, dimension(:)   , allocatable	        :: tmp, kk

  integer                                       :: na, ia
  real                                          :: a0


  call initpar()
  call sep_init(SOURCE)
  ndim=sep_dimension()

  allocate(n(ndim)) 
  call from_history('n',n)

  n1=n(1)
  n2=n(2)
  nd=n2

  if (n1 /= 3) call erexit ('I dont even like you.  Get lost.')

  allocate (xy (nd,2))
  allocate (z  (nd))
  allocate (input(3,nd))

  call sep_read(input)

  xy(:,1) = input(1,:) !x coordinate
  xy(:,2) = input(2,:) !y coordinate
  z(:)    = input(3,:) !z depth

  ! get model dimensions from pars

  call from_param('rho',rho,1.)
  call from_param('m1',mn1)
  call from_param('m2',mn2)
  call from_param('o1',mo1)
  call from_param('o2',mo2)
  call from_param('d1',md1)
  call from_param('d2',md2)
  call from_param('ncoef',ncoef,16)
  call from_param('lapeps',lapeps,0.0001)
  call from_param('tau',tau,1.)

  aa = lapfac2(lapeps,mn1,ncoef)

  write(0,*) '------------------------'
  write(0,*) '*      PARAMETERS      *'
  write(0,*) '------------------------'
  write(0,*) '    rho =    ',rho
  write(0,*) '    mn1 =    ',mn1
  write(0,*) '    mn2 =    ',mn2
  write(0,*) '    mo1 =    ',mo1
  write(0,*) '    mo2 =    ',mo2
  write(0,*) '    md1 =    ',md1
  write(0,*) '    md2 =    ',md2
  write(0,*) '------------------------'
  write(0,*) '+'

  ! put model dimensions in output

  call to_history ("n1",mn1)
  call to_history ("o1",mo1)
  call to_history ("d1",md1)
  call to_history ("n2",mn2)
  call to_history ("o2",mo2)
  call to_history ("d2",md2)
  call to_history ("n3",  1)

  ! allocate model space, model gradient, and residual if it exists

  if (exist_file ("residual")) then
     write(0,*) '-------------------'
     write(0,*) 'Found file residual'
     write(0,*) '+'
     call to_history ("n1",mn1,"residual")
     call to_history ("o1",mo1,"residual")
     call to_history ("d1",md1,"residual")
     call to_history ("n2",mn2,"residual")
     call to_history ("o2",mo2,"residual")
     call to_history ("d2",md2,"residual")
     call to_history ("n3",  3,"residual")
  end if

  if (exist_file ("tide")) then
     write(0,*) '---------------'
     write(0,*) 'Found file tide'
     write(0,*) '---------------'
     write(0,*) '+'
     call to_history ("n1", 5718,"tide")
     call to_history ("o1", 1,   "tide")
     call to_history ("d1", 1,   "tide")
     call to_history ("n2", 23,  "tide")
     call to_history ("o2", 1,   "tide")
     call to_history ("d2", 1,   "tide")
     call to_history ("n3", 3,   "tide")
  end if
       
  m12 = mn1 * mn2
  allocate (tmp(m12))
  allocate (kk(size(z)))

  ! initialize interpolation operator

  call  lint_helderiv_init(m12)  
  call        polydiv_init(m12,aa)
  call track_modeling_init(m12,tau)
  call          lint2_init(mn1,mn2,mo1,md1,mo2,md2,xy)
  call        leakint_init(rho)

  call from_param ("niter",niter,1)
  call from_param ("nfreq",nfreq,niter)
  call from_param ("nmem",nmem,0)
  call from_param ("l1cst",l1cst,maxval(z)*2) 
  call from_param ("inv",inv,0)

  write(0,*) '------------------------'
  write(0,*) '* INVERSION PARAMETERS *'
  write(0,*) '------------------------'
  write(0,*) '    niter = ',niter
  write(0,*) '    nfreq = ',nfreq
  write(0,*) '    nmem  = ',nmem
  write(0,*) '    l1cst = ',l1cst
  write(0,*) '      tau = ',tau
  write(0,*) '      inv = ',inv
  write(0,*) '------------------------'
  write(0,*) '+'

  stat = l1_priv_init(l1cst)
  
  if (inv.eq.1) then

     write(0,*) " With Leaky integration " 
     allocate (x(m12+size(z)))      ;  x=0.
     call solver_irls(Fop=track_modeling_op, stepper=cgstep, niter=niter, m=x, &
     d=z , Wdiag=l1_priv, nmem=nmem, nfreq=nfreq, verb=.true.)

  else if (inv.eq.2) then

     write(0,*) " Without Leaky integration but with d/ds " 
     allocate (x(m12))      ;  x=0.
     call solver_irls(Fop=lint_helderiv_op, stepper=cgstep, niter=niter, m=x, &
     d=z, Wop=igrad1_lop, Wdiag=l1_priv, nmem=nmem, nfreq=nfreq, verb=.true.)

  else

     write(0,*) " Without Leaky integration "
     allocate (x(m12))      ;  x=0.
     call solver_irls(Fop=lint_helderiv_op, stepper=cgstep, niter=niter, m=x, &
     d=z, Wdiag=l1_priv, nmem=nmem, nfreq=nfreq, verb=.true.)

  end if

  if (exist_file("residual")) then

     allocate(rd(size(z)))
     allocate(rm(m12))
     rd   = -z

     if (inv.eq.1) then
        stat = track_modeling_op(F,T,x,rd)
     else
        stat = lint_helderiv_op(F,T,x,rd)
     end if
     
     stat = l1_priv(rd,kk)
     call sep_write(rd,"tide")
     rd   = kk*rd
     stat = lint2_lop(T,F,rm,rd)
     
     call sep_write(rm,"residual")
     call sep_write(rd,"tide")
     
     
     stat = leakint_lop(F,F,x(m12+1:),rd)
     call sep_write(tau*rd,"tide")
     stat = lint2_lop(T,F,rm,rd)
     
     call sep_write(tau*rm,"residual")
     
     stat = lint2_lop(T,F,rm,-z)
     call sep_write(rm,"residual")
     

     deallocate(rd,rm)

  end if

  stat=polydiv_lop(F,F,x(1:m12),tmp)

  call cgstep_close ()
  call lint_helderiv_clean()
  call sep_write(tmp)
  deallocate(n,z,xy,input,x,tmp,kk)

  call exit (0)

end program Galilee









