module deburst1
  use irls
  use copy
  use icai1
  use triangle
  use solver_mod 
  use cgstep_mod

  implicit none

contains

  subroutine deburst ( niter, p, eps, dd, hh)
    integer, intent (in)              :: niter, p
    real, intent (in)                 :: eps
    real, dimension (:), intent (in)  :: dd 
    real, dimension (:), intent (out) :: hh

    real, dimension (:), pointer      :: ff
    integer                           :: nd

    nd = size (dd) 
    allocate (ff (3)) ; ff = (/ -eps, 2*eps, -eps /)  
    call icai1_init (ff, nd)
    call cgstep_init ()

    select case (p)
       case (0)
          call solver_reg (copy_lop, cgstep, wght = cauchy, nfreq = 1, nmem = 0, &
          reg = icai1_lop, nreg = nd, niter = niter, x = hh, dat = dd,eps=1.)
       case (1)
          call solver_reg (copy_lop, cgstep, wght = l1,     nfreq = 1, nmem = 0, &
          reg = icai1_lop, nreg = nd, niter = niter, x = hh, dat = dd, eps=1.)
       case default
          call triangle_init (p, nd)
          call solver_prec (copy_lop, cgstep, wght = cauchy,  nfreq = 1, nmem = 0, &
          prec = triangle_lop, nprec = nd, niter = niter, x = hh, dat = dd, eps = eps)
          call triangle_close ()
    end select

    call cgstep_close ()
    deallocate (ff)
  end subroutine deburst

end module deburst1
