program GALILEE
  use sep
  use int2
  use interp
  use medbin2_mod

  implicit none
  integer                                       :: n2, nd, stat, median
  integer, dimension (2)                        :: m
  real,    dimension (:),   allocatable         :: z, mm, ones, cont
  real,    dimension (:),   pointer             :: x, y
  real,    dimension (:,:), allocatable, target :: coord
  real,    dimension (2)                        :: o, d, cmin, cmax
  real                                          :: dx

  call sep_init ()

  ! create coordinates
  call from_history (nd, n2)
  if (n2 /= 3) call erexit ('bad n2')
  allocate (coord (nd,2))
  call sep_read (coord)
  x => coord (:,1)
  y => coord (:,2)

  ! create ordinates
  allocate (z (nd))
  call sep_read (z)

  ! create model
  call from_par ("n1", m (1), 200)
  call from_par ("n2", m (2), 200)

  call to_history ("n1", m (1))
  call to_history ("n2", m (2))
  call to_history ("n3", 1)

  call from_par ("xmin", cmin (1), minval (x))
  call from_par ("ymin", cmin (2), minval (y))
  call from_par ("xmax", cmax (1), maxval (x))
  call from_par ("ymax", cmax (2), maxval (y))

  dx = sum ((cmax - cmin) / (m - 1)) * 0.5

  call from_par ("o1", o (1), cmin (1))
  call from_par ("o2", o (2), cmin (2))
  call from_par ("d1", d (1), dx)
  call from_par ("d2", d (2), dx)

  ! initialize interpolation
  call from_par ("median", median, 0)
  call sep_close ()

  if (median == 0) then
     call int2_init (coord, o, d, m, bin2_int,  (/ 1, 1 /), nd)
  else
     call medbin2_init (coord, o, d, m)
  end if
  deallocate (coord)

  ! fill model
  allocate ( mm(m(1)*m(2)) )

  if (median == 0) then
     allocate (ones (nd), cont (size (mm))) ; ones = 1.
     stat = int2_lop (.true.,.false.,mm,z)
     stat = int2_lop (.true.,.false.,cont,ones)
     where (cont > 0.) mm = mm / cont
     call int2_close ()
     deallocate (ones, cont)
  else
     call medbin2 (z, mm)
     call medbin2_close ()
  end if

  call sep_write (mm)
  deallocate (z, mm)
  call exit (0)
end program GALILEE







