program Rufftri2
  ! DESCRIPTION
  !      Rufftri2 2-D with triangle
  !
  ! USAGE
  !      <in.h  Rufftri2 [options] > out.H
  !
  ! INPUT PARAMETERS
  !  rect1    - integer [1]	halfwidth on 1-axis  (=1 means no smoothing);
  !  rect2    - integer [1]	halfwidth on 2-axis  (=1 means no smoothing)
  !                     halfwidth is the width in sample points of 
  !                     a rectangle which when convolved with itself 
  !                     gives a triangle.
  ! agc       - integer [0] =1  to use smoothed data as 
  !                      divisor gain for data. (UNTESTED!)
  ! absval    - integer [agc]  =1 means smooth absolute values of input.
  ! scaleup   - integer [0]   =1 means scale up to compensate for bandwidth 
  !                         reduction. (bug?)
  use sep
  use triangle2
  use ruffen2
  use leaky2

  implicit none
  integer                               :: n12, stat, sum3, i3, n3, ruffen
  integer	                        :: scaleup, absval, agc
  integer, dimension (2)                :: n, nrect
  real,    dimension (:),   allocatable :: data, summ, smoo, tmp
  real,    dimension (2)                :: rect, d

  call sep_init ()
  call from_history (n (1))
  call from_history ("n2",n (2),1) ; n12 = n (1) * n (2)
  call from_history ("n3",n3,1)

  call from_history ("d1", d (1)) ; call from_par ("nrect1", nrect (1), 1) 
  call from_history ("d2", d (2)) ; call from_par ("nrect2", nrect (2), 1) 

  rect = nrect*d
  call from_par ("rect1", rect (1), rect (1))
  call from_par ("rect2", rect (2), rect (2))
  nrect = .5 + rect/d

  call to_history ("nrect1", nrect (1))
  call to_history ("nrect2", nrect (2))

  call from_par ("scaleup", scaleup, 0)
  call from_par ("ruffen", ruffen, 1)

  call from_par ("agc", agc, 0)
  call from_par ("absval", absval, agc)
  call from_par ("sum3", sum3, 0)

  if (agc > 0  .and.  sum3 > 0) call erexit('I cannot do both agc>0 and sum3>0')
  if (agc > 0)	                call putlin('AGC:  Automatic Gain Control')
  if (absval > 0)	        call putlin('Absolute values of data used.')
  call putlin('Data smoothed with triangle().')
  if (ruffen>0)  call putlin('Data roughened after smooth.')
  call to_history ('Data smoothed with: rect1',rect (1))
  call to_history ('Data smoothed with: rect2',rect (2))
  if (sum3 > 0 ) call to_history ("n3",1)
  call sep_close ()	

  allocate (data(n12), smoo(n12), tmp (n12))

  if  (sum3  > 0) then
     allocate (summ (n12)) ; summ = 0.
  end if

  select case (ruffen)
  case (0)
     call triangle2_init (nrect,n)
  case (-1)
     call leaky2_init (rect, n)
  case default
     call   ruffen2_init (nrect,n)
  end select

  do i3= 1, n3 
     call sep_read (data) ; if (absval > 0) data = abs (data)

     select case (ruffen)
     case (0) 
        stat = triangle2_op (.true., .false., smoo, data)
     case (-1)
        stat = leaky2_op (.true., .false., smoo, data)
     case default
        stat =   ruffen2_op (.true., .false., smoo, data)
     end select

     if (scaleup > 0) smoo = smoo/(rect (1)**2 * rect (2)**2)

     if (agc > 0) where (smoo /= 0.) smoo = data / smoo

     if (sum3 == 0) then
        call sep_write (smoo)
     else
        summ = summ + smoo
     end if
  end do

  select case (ruffen)
  case (0) 
     call triangle2_close ()
  case (-1)
     call leaky2_close ()
  case default
     call   ruffen2_close ()
  end select

  if  (sum3  /= 0) then
     call sep_write (summ)
     deallocate (summ)
  end if

  deallocate (data, smoo, tmp)
  call exit (0)
end program Rufftri2
