module pefest_mod
  use quantile_mod
  use icai1
  use pef_mod

  implicit none

contains

  ! Estimate a PEF avoiding zeros and bursty noise on input.
  !
  subroutine pefest (niter, yy, aa)
    integer, intent (in)    :: niter
    real,    dimension (:), pointer  :: yy, aa

    real,    dimension (size (yy))   :: rr, yfre
    real,    dimension (:), pointer  :: bb
    logical, dimension (:), pointer  :: mx, ma
    real, parameter                  :: easilygood = .33
    real                             :: rbar
    integer                          :: stat, nr, na

    nr = size (rr)
    na = size (aa)

    allocate (mx(nr), ma (na))

    allocate (bb (3)) ; bb = (/ 1., -2., 1. /)   ! starting guess
    call icai1_init (bb, nr)
    stat = icai1_op (.false., .false., yy, rr)

    rr = abs (rr) 
    rbar = quantile (easilygood, rr)             ! rbar={ r safe below rbar}

    where (yy == 0. .or. rr > 5 * rbar)
       yfre = 1.                                 ! risky
    elsewhere
       yfre = 0.                                 ! safe enuf
    end where
    yfre (1 : 2) = 1.

    aa = 1.
    call icai1_init (aa, nr)
    stat = icai1_op (.false., .false., yfre, rr) ! likely bad outputs.

    mx = (rr <= 0.)                              ! safe   regressors
    stat = count (mx)                            ! stupid SGI's compiler

    ma = .false.   ; ma (1) = .true.
    aa = 0.        ; aa (1) = 1.
    call find_pef (yy, aa, na, 1, mx, ma, niter)

    deallocate (bb, mx, ma)
  end subroutine pefest

end module pefest_mod



